#!/bin/sh
#
# test2.bat
# tests aligning phonological reps
# 
tald3e_sm         \
  xp_out.aln      \
  xp_in.aln       \
  pcdt_ph.txt     \
  dummy_ttp_rls   \
  ./data/nat+for  \
  1.25            \
  20000           \
  0               \
 > test2.rpt