/* file swcase.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

char *switch_case(char *sx)
 /***********************************/
 /* Switches case of letters in *sx */
 /***********************************/
 {char *pa;
  for (pa=sx; *pa != '\0'; pa++)
    {if (islower(*pa)) *pa = toupper(*pa);
     else              *pa = tolower(*pa);
    }
  return sx;
 }
