/* file gp/sylbcons.h */
#if !defined(SYLB_CONSTANTS)
#define SYLB_CONSTANTS
#define MAX_SYLB_PRONS 256
#define MAX_SYLBS       24
#define MAX_SEGS       120
#define MAX_LECTS       12
#define MAX_RATE         5
#define MIN_RATE         1
#endif
