 /* File initaln2_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

 void init_al_int_prot(ALIGNMENT_INT_PROT *paln1, pcodeset *pcode1)
 /**********************************************************************/
 /*                                                                    */
 /* Initializes the protected integer array alignment *paln1 with      */
 /*   pcodeset *pcode1.  Other items are set to legitimate             */
 /*   but informationless values.                                      */
 /*                                                                    */
 /**********************************************************************/

 {
/* local data: */
  char *proc = "init_al_int_prot";
/* code: */
 db_enter_msg(proc,1); /* debug only */
  paln1->aligned = T;
  paln1->d = 0;
  paln1->nsa->str[0] = 0;
  paln1->nsb->str[0] = 0;
  paln1->ia = 0;
  paln1->jb = 0;
  paln1->pcset = pcode1;
  paln1->naux = 0;
 db_leave_msg(proc,1); /* debug only */
 } /* end of function "init_al_int_prot" */
/* end of file initaln2_prot.c */
