 /* file fralchr_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

 void free_al_chr_prot(ALIGNMENT_CHR_PROT *p)
 /**********************************************************************/
 /*                                                                    */
 /* Frees all memory space associated with a protected character-string*/
 /* alignment *p                                                       */
 /**********************************************************************/
  {char *proc = "free_al_chr_prot";
 db_enter_msg(proc,3); /* debug only */
   destroy_prot_chr_str(p->id);
   destroy_prot_chr_str(p->sa);
   destroy_prot_chr_str(p->sb);
   free((char*)p);
 db_leave_msg(proc,3); /* debug only */
   return;
  }
