/* file spmode_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

PROTECTED_CHR_STR  *spell_mode_prot(PROTECTED_CHR_STR *sph, SUBSTRING *ss_token)
/*************************************************************/
/* Returns "spell mode" pronunciation of orthographic token  */
/* *ss_token in protected character string *sph.             */
/*************************************************************/
  {char *proc = "spell_mode_prot";
   char *p;
   PROTECTED_CHR_STR *psx;
 db_enter_msg(proc,1); /* debug only */
   psx = create_prot_chr_str(133);
   sph = strcpy_prot_s(sph,"");
   for (p = ss_token->start; p <= ss_token->end; p++)
     {sph = strcat_prot_p(sph,char_name_prot(psx,*p));
     }
   destroy_prot_chr_str(psx);
 db_leave_msg(proc,1); /* debug only */
   return sph;
  }
