 /* file fralint_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif


 void free_al_int_prot(ALIGNMENT_INT_PROT *pal)
 /**********************************************************************/
 /*                                                                    */
 /* Frees all memory space associated with an integer alignment *p     */
 /* NOTE: should NOT be used if p->aint has been set to point into a   */
 /* pcodeset!!!!                                                       */
 /**********************************************************************/
  {char *proc = "free_al_int_prot";
   int i;
 db_enter_msg(proc,3); /* debug only */
   destroy_prot_int_str(pal->nsa);
   destroy_prot_int_str(pal->nsb);
   if (pal->naux > 0)
     {for (i=1; i <= pal->naux; i++)
        {if (memory_trace) printf("%s FREE %x\n",pdb,(int)pal->auxtab[i]);
         destroy_prot_int_str(pal->auxtab[i]);
     }  }
if (db_level > 3) printf("%s freeing %s->auxtab at %x\n",pdb,pal->id->str,(int)pal->auxtab);
   free((char*)pal->auxtab);
   if (memory_trace) printf("%s FREE %x\n",pdb,(int)pal);
   destroy_prot_chr_str(pal->id);
   free((char*)pal);
 db_leave_msg(proc,3); /* debug only */
   return;
  }
