 /* File ttpparms.h --  TTP parameters for ttp3s() */
#ifndef TTP_PARMS_STR1_HEADER
#define TTP_PARMS_STR1_HEADER

typedef struct
  {char *desc;
   pcodeset *pcdrls;
   struct BPATH *bpath;
   struct SYL_PARMS *syl_parms;
  } TTP_PCDRLS_PARAMETERS;

#endif
