#!/bin/sh
#
# test1.bat
# tests if tald3e_sm is working o.k. or not
# (when done, do "diff x_sm.aln x_sm_NIST.aln")
# 
tald3e_sm                \
  x_sm.aln               \
  x.aln                  \
  pcdt_eng_p.txt         \
  ./data/ttp411_l3r4.rls \
  ./data/nat+for         \
  1.25                   \
  20000                  \
  0                      \
 > test1.rpt
