/* file bpath.h */
/* struct holding a backoff path */

 struct RPARTS {int le; int tg; int re;};
 struct BPATH
   {char *desc;
    struct RPARTS rpart[MAX_BACKOFF_STEPS];
    int step1;
    int stepn;
   };
