/* file alstr_chr_prot.h */

#ifndef ALSTR_CHR_PROT_HEADER
#define ALSTR_CHR_PROT_HEADER

 typedef struct
   {/* a basic alignment object using protected character strings */
    PROTECTED_CHR_STR *id; /* identification */
    boolean aligned; /* are the strings aligned, or not? */
    int d; /* alignment distance, if they are aligned */
    PROTECTED_CHR_STR *sa;  /* "a" string */
    PROTECTED_CHR_STR *sb;  /* "b" string */
/* for future context-sensitive work: */
    char *pia;  /* pia and pjb point out a particular symbol-symbol match */
    char *pjb;  /* pia indexes into sa->str[], pjb into sb->str[].  */
   } ALIGNMENT_CHR_PROT;

#endif
