/* file fsylparms2.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

void free_syl_parms2(struct SYL_PARMS *syl_parms)
  /**************************************************************/
  /*  Frees allocated space in syllabification parameters       */
  /*   except for the pcodeset pointed to and the directories.  */
  /**************************************************************/
 {char *proc = "free_syl_parms2";
  int i;
 /* code */
db_enter_msg(proc,1); /* debug only */
  free(syl_parms->desc);
/*  free(syl_parms->data_dir);  */
/*  free(syl_parms->pcdp_dir);  */
/*  free_pcode(syl_parms->pcdp);*/
/*  free(syl_parms->pcdp);      */
  for (i=1; i <= syl_parms->n_CC_inits; i++)     free(syl_parms->CC_init[i]);
  for (i=1; i <= syl_parms->n_CC_bad_inits; i++) free(syl_parms->CC_bad_init[i]);
  for (i=1; i <= syl_parms->n_CC_final; i++)     free(syl_parms->CC_final[i]);
db_leave_msg(proc,1); /* debug only */
  return;
 }
