 /* File initalc2_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

 /**********************************************************************/
 /*                                                                    */
 /* Initializes the protected character string alignment *palc1 to     */
 /* legitimate but informationless values.                             */
 /*                                                                    */
 /**********************************************************************/

 void init_al_chr_prot(ALIGNMENT_CHR_PROT *palc1)
{
/* local data: */
  char *proc = "init_al_chr_prot";
/* code: */
 db_enter_msg(proc,3); /* debug only */
  strcpy_prot_s(palc1->sa,"");
  strcpy_prot_s(palc1->sb,"");
  palc1->pia = palc1->sa->str;
  palc1->pjb = palc1->sb->str;
  palc1->aligned = T;
  palc1->d = 0;
 db_leave_msg(proc,3); /* debug only */
  return;
  } /* end of function "init_al_chr_prot" */
