/********************************************/
/* file stdcenvp.h                          */
/* standard compile environment for progs   */
/********************************************/

#define COMPILE_ENVIRONMENT 1

#include "stdio.h"
#include "ctype.h"
#include "limits.h"
#include "math.h"
#include "string.h"
#include "time.h"
#include "stdlib.h"
#include "sys/stat.h"

/* standard program parameter settings */
#include "stdpars.h"
#include "sylbcons.h"

/* type definitions */
#include "edits_enum.h"
#include "booldef.h"
#include "wchardef.h"
#include "num_rel.h"
#include "interv1.h"
#include "prot_chr_str.h"
#include "prot_int_str.h"
#include "pcinddef.h"
#include "hparstr1.h"
#include "rulestr2.h"
#include "pcstr3.h"
#include "ppfstr2.h"
#include "pctabstr.h"
#include "ssstr1.h"
#include "alstr_chr_prot.h"
#include "alstr_int_prot.h"
#include "smstr2.h"
#include "pron2.h"
#include "syl_parms.h"
#include "bpath.h"
#include "ttpparms.h"

/* macros */
#include "genmacs.h"
#include "strmacs.h"
#include "array_alloc_macs.h"

/* global data definitions (NOT just declarations) */
#include "gdatadef.h"

/* function declarations */
#include "fcndcls.h"
