/* file dauxtab_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

void dump_auxtab_prot(ALIGNMENT_INT_PROT *pal)
/*************************************************/
/*  dumps the auxiliary pcodes in pla->auxtab.   */
/*************************************************/
 {int ix,jx,iaux;
  printf("\n");
  printf(" DUMP OF ALIGNMENT '%s' PCODE AUXTAB:\n",pal->id->str);
  printf(" naux = %d\n",pal->naux);
  if (pal->naux > 0)
    {printf("   i    iaux nsymbs auxtab[i][*]-->\n");
     for (ix = 1; ix <= pal->naux; ix++)
       {iaux = pal->pcset->pctab_length + ix;
        printf(" %3d   %3d %6d",ix,iaux,pal->auxtab[ix]->str[0]);
        for (jx = 1; jx <= pal->auxtab[ix]->str[0]; jx++)
           printf(" %5d",pal->auxtab[ix]->str[jx]);
        printf("\n");
       }
    }
  printf("\n");
  return;
 }
/* end function dump_auxtab_prot() */
