/* file fbpath1.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

void free_bpath1(struct BPATH *bpath)
 
  /***********************************************/
  /*  Frees allocated space in back-off path.    */
  /***********************************************/
 {char *proc = "free_bpath1";
 /* code */
db_enter_msg(proc,1); /* debug only */
  free(bpath->desc);
db_leave_msg(proc,1); /* debug only */
  return;
 }
