/* file char_name_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

PROTECTED_CHR_STR *char_name_prot(PROTECTED_CHR_STR *psx, char character)
/**********************************************************/
/* returns English pronunciation of character name in *psx */
/**********************************************************/
{switch (character)
   {case 'a': strcpy_prot_s(psx," '1 ey");                     break;
    case 'b': strcpy_prot_s(psx," b '1 iy");                   break;
    case 'c': strcpy_prot_s(psx," s '1 iy");                   break;
    case 'd': strcpy_prot_s(psx," d '1 iy");                   break;
    case 'e': strcpy_prot_s(psx," '1 iy");                     break;
    case 'f': strcpy_prot_s(psx," '1 eh f");                   break;
    case 'g': strcpy_prot_s(psx," jh '1 iy");                  break;
    case 'h': strcpy_prot_s(psx," '1 ey ch");                  break;
    case 'i': strcpy_prot_s(psx," '1 ay");                     break;
    case 'j': strcpy_prot_s(psx," jh '1 ey");                  break;
    case 'k': strcpy_prot_s(psx," k '1 ey");                   break;
    case 'l': strcpy_prot_s(psx," '1 eh l");                   break;
    case 'm': strcpy_prot_s(psx," '1 eh m");                   break;
    case 'n': strcpy_prot_s(psx," '1 eh n");                   break;
    case 'o': strcpy_prot_s(psx," '1 ow");                     break;
    case 'p': strcpy_prot_s(psx," p '1 iy");                   break;
    case 'q': strcpy_prot_s(psx," k y '1 uw");                 break;
    case 'r': strcpy_prot_s(psx," '1 aa r");                   break;
    case 's': strcpy_prot_s(psx," '1 eh s");                   break;
    case 't': strcpy_prot_s(psx," t '1 iy");                   break;
    case 'u': strcpy_prot_s(psx," y '1 uw");                   break;
    case 'v': strcpy_prot_s(psx," v '1 iy");                   break;
    case 'w': strcpy_prot_s(psx," d '1 ax b '0 ax l y '2 uw"); break;
    case 'x': strcpy_prot_s(psx," '1 eh k s");                 break;
    case 'y': strcpy_prot_s(psx," w '1 ay");                   break;
    case 'z': strcpy_prot_s(psx," z '1 iy");                   break;
    case '0': strcpy_prot_s(psx," z '1 iy r '0 ow");   break;
    case '1': strcpy_prot_s(psx," w '1 ax n");         break;
    case '2': strcpy_prot_s(psx," t '1 uw");           break;
    case '3': strcpy_prot_s(psx," th r '1 iy");        break;
    case '4': strcpy_prot_s(psx," f '1 ow r");         break;
    case '5': strcpy_prot_s(psx," f '1 ay v");         break;
    case '6': strcpy_prot_s(psx," s '1 ih k s");       break;
    case '7': strcpy_prot_s(psx," s '1 eh v '0 ax n"); break;
    case '8': strcpy_prot_s(psx," '1 ey t");           break;
    case '9': strcpy_prot_s(psx," n '1 ay n");         break;
    case '&': strcpy_prot_s(psx," '0 ae n d");         break;
    default: strcpy_prot_s(psx,"");
   }
  return psx;
 }
