/* file syl_parms.h */
/* struct holding syllabification parameters */
#if !defined(MAX_CC)
#define MAX_CC 500
#endif

 struct SYL_PARMS
   {char *desc;
    char *data_dir;
    char *pcdp_dir;
    pcodeset *pcdp;
     /* i.d.s of some special phonemes: */
    int lmark[2], rmark[2]; /* left & right sylb boundaries (int str) */
    int wbound[2]; /* word boundary (int str) */
    int istr_1, istr_2, istr_0; /* primary, secondary, zero stress (int) */
    int iwordbound; /* word boundary (int) */
    int i_default_stress; /* default stress mark (int) */
   /* tables of allowable consonant clusters: */
    int n_CC_inits, n_CC_bad_inits,n_CC_final;
    int *CC_init[MAX_CC], *CC_bad_init[MAX_CC], *CC_final[MAX_CC];
   };
