/* file dbpaths.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

void dump_bpaths(struct BPATH *bpath) 
  /*****************************************************/
  /*  Dumps the backoff path trajectory                */
  /*****************************************************/
 {char *proc = "dump_bpaths";
  int i;
/* code */
db_enter_msg(proc,2); /* debug only */
  printf("\n");
  printf(" DUMP OF BACKOFF PATH\n");
  printf(" bpath->desc = '%s'\n",bpath->desc);
  printf(" bpath->step1 = %d\n",bpath->step1);
  printf(" bpath->stepn = %d\n",bpath->stepn);
  if ((bpath->step1 > 0)&&(bpath->step1 <= MAX_BACKOFF_STEPS)
    &&(bpath->stepn > 0)&&(bpath->stepn <= MAX_BACKOFF_STEPS)
    &&(bpath->step1 <= bpath->stepn))
    {printf("  I  LE  TG  RE\n");
     for (i=bpath->step1; i <= bpath->stepn; i++)
       {printf(" %2d  %2d  %2d  %2d\n",i,
        bpath->rpart[i].le,bpath->rpart[i].tg,bpath->rpart[i].re);
       }
     printf("\n");
    }
  else printf(" No path -- step1=%d, stepn=%d\n",bpath->step1,bpath->stepn);
db_leave_msg(proc,2); /* debug only */
 return;
}
