/* file dsylparms.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

void dump_syl_parms(struct SYL_PARMS *syl_parms)
  /**************************************************************/
  /*  Dumps parameter values needed for syllabification.        */
  /**************************************************************/
 {char *proc = "dump_syl_parms";
 /* code */
db_enter_msg(proc,1); /* debug only */
  printf("%s\n",pdb);
  printf("%s DUMP OF SYLLABIFICATION PARAMETERS:\n",pdb);
  printf("%s data_dir = '%s'\n",pdb,syl_parms->data_dir);
  printf("%s pcdp_dir = '%s'\n",pdb,syl_parms->pcdp_dir);
  printf("%s pcdp = '%s'\n",pdb,syl_parms->pcdp->name);
  printf("%s lmark[1] = %d(%s)\n",pdb,syl_parms->lmark[1],
   syl_parms->pcdp->pc[syl_parms->lmark[1]].ac);
  printf("%s rmark[1] = %d(%s)\n",pdb,syl_parms->rmark[1],
   syl_parms->pcdp->pc[syl_parms->rmark[1]].ac);
  printf("%s wbound[1] = %d(%s)\n",pdb,syl_parms->wbound[1],
   syl_parms->pcdp->pc[syl_parms->wbound[1]].ac);
  printf("%s iwordbound = %d(%s)\n",pdb,syl_parms->iwordbound,
   syl_parms->pcdp->pc[syl_parms->iwordbound].ac);
  printf("%s i_default_stress = %d(%s)\n",pdb,syl_parms->i_default_stress,
    syl_parms->pcdp->pc[syl_parms->i_default_stress].ac);
  printf("%s n_CC_inits=%d\n",pdb,syl_parms->n_CC_inits);
  printf("%s n_CC_bad_inits=%d\n",pdb,syl_parms->n_CC_bad_inits);
  printf("%s n_CC_final=%d\n",pdb,syl_parms->n_CC_final);
db_leave_msg(proc,1); /* debug only */
  return;
 }
