/* file dintal2_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

 void dump_int_alignment_prot(ALIGNMENT_INT_PROT *p1)
/************************************************************/
/*  dumps *p1, an ALIGNMENT_INT_PROT                        */
/************************************************************/
 {char *proc = " dump_int_alignment_prot";
  ALIGNMENT_CHR_PROT *p2;
  int err, *perr = &err;
 db_enter_msg(proc,4); /* debug only */
  p2 = make_al_chr_prot2(p1->id->str);
  aln_i_to_s_prot5(p1,p2," | ",perr);
  if (*perr > 0)
    {fprintf(stdout,"%s: converting to s alignment err=%d\n",proc,*perr);
     goto RETURN;
    }
  dump_chr_alignment_prot(p2);
RETURN:
  free_al_chr_prot(p2);
 db_leave_msg(proc,4); /* debug only */
  return;
 } /* end dump_int_alignment */
 /* end file dintal2.c */
