/* file wprob.h */
/* Data structures for word/probabilities and lists of them */

  struct WPROB
    {int word_id;
     double prob;
    };

  struct WPROB_LIST_ATOM
    {struct WPROB *word_pr;
     struct WPROB_LIST_ATOM *next;
    };

