 /* file malchr_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif


 ALIGNMENT_CHR_PROT *make_al_chr_prot2(char *ident)
 /**********************************************************************/
 /*                                                                    */
 /* Allocates memory space for an object of type ALIGNMENT_CHR_PROT    */
 /* a protected character string alignment) and initializes it with    */
 /* legitimate but informationless values.  Returns a pointer to the   */
 /* alignment.                                                         */
 /*                                                                    */
 /**********************************************************************/
{
/* local data: */
  char *proc = "make_al_chr_prot2";
 ALIGNMENT_CHR_PROT *p;
/* code: */
 db_enter_msg(proc,2); /* debug only */
  p = (ALIGNMENT_CHR_PROT *) malloc_safe(sizeof(ALIGNMENT_CHR_PROT),proc);
  p->id = create_prot_chr_str(65);
  p->sa = create_prot_chr_str(2001);
  p->sb = create_prot_chr_str(2001);
  strcpy_prot_s(p->id,ident);
  init_al_chr_prot(p);
 db_leave_msg(proc,2); /* debug only */
  return p;
  } /* end of function "make_al_chr_prot" */
/* end file malchr_prot.c */
