/* file dchral_prot.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

 void dump_chr_alignment_prot(ALIGNMENT_CHR_PROT *p1)
/************************************************************/
/*  dumps *p1, an alignment of protected character strings. */
/************************************************************/
{char *proc = " dump_chr_alignment_prot";
db_enter_msg(proc,4); /* debug only */
  fprintf(stdout,"id: %s\n",p1->id->str);
  if (p1->aligned) printf(" The sequences are aligned.\n");
  else             printf(" The sequences are NOT aligned.\n");
  fprintf(stdout,"a: %s\n",p1->sa->str);
  fprintf(stdout,"b: %s\n",p1->sb->str);
  fprintf(stdout,"\n");
db_leave_msg(proc,4); /* debug only */
  return;
 } /* end dump_chr_alignment */

 /* end file dchral.c */
