/* file frele.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

  /************************************************************/
  /*   free_pcode_element(px)                                 */
  /*   frees the allocated space in the pcode element *px.    */
  /*   error codes:                                           */
  /*       0: a.o.k.                                          */
  /************************************************************/
  void free_pcode_element(pcode_element *px)
   {char *proc = "free_pcode_element";
  /* initialize */
 db_enter_msg(proc,3); /* debug only */
    if (strlen(px->ac) > 0) free(px->ac);
    if ((px->lc_ext != NULL)&&(px->lc_ext != null_char_string)) free(px->lc_ext);
    if (px->lc_int != NULL) free_str((char*)px->lc_int);
 db_leave_msg(proc,3); /* debug only */
    return;
  } /* end free_pcode_element */
