/* file rdiacrit.c */

#if !defined(COMPILE_ENVIRONMENT)
#include <gp/stdcenvf.h> /* std compile environment for functions */
#endif

char *remove_diacritics(char *out, char *in)
/*********************************************/
/* Process input text, removing diacritics.  */
/*********************************************/
{char *p, *q;
 q = out;
 for (p = in; *p != '\0'; p++)
   {if ((*p == '\\')&&(isdigit(*(p+1)))) p++; /* skip this & next */
    else *q++ = *p;
   }
 *q = '\0';
 return out;
}
